var model = null;

function MRRemoteModel() {
	var self = this;
	
	self.sources = ko.observableArray();
	self.destinations = ko.observableArray();
	self.viewers = ko.observableArray();
	self.numberOfViewersSelected = ko.observable(0);
	
	self.canGangStop = ko.computed(function() {
		var nbOfselectedSourcesRecording = 0;
		for(var i = 0; i < self.viewers().length; i++) {
			if (self.viewers()[i].isSelected() && self.viewers()[i].selectedSource() && self.viewers()[i].selectedSource().isRecording()) {
				return true;
			}
		}
		return false;
	}, self);
	self.canGangPause = ko.computed(function() {
		var nbOfselectedSourcesRecording = 0;
		for(var i = 0; i < self.viewers().length; i++) {
			if (self.viewers()[i].isSelected() && self.viewers()[i].selectedSource() && self.viewers()[i].selectedSource().isRecording() && self.viewers()[i].selectedSource().isPaused() === false) {
				return true;
			}
		}
		return false;
	}, self);
	self.canGangRecord = ko.computed(function() {
		var nbOfselectedSourcesRecording = 0;
		for(var i = 0; i < self.viewers().length; i++) {
			if (self.viewers()[i].isSelected() && self.viewers()[i].selectedSource() && (self.viewers()[i].selectedSource().isRecording() === false || self.viewers()[i].selectedSource().isPaused() === true)) {
				return true;
			}
		}
		return false;
	}, self);
}

function Viewer() {
	var self = this;
	self.selectedSource = ko.observable(null);
	self.isSelected = ko.observable(false);
	self.previewImage = null;
	self.shouldDraw = false;
	self.shouldFetch = false;
	self.animationRequestID = null;
	self.context = null;
	
	self.isRecording = ko.computed(function() {
		return (self.selectedSource() && self.selectedSource().isRecording);
	});
	self.selectViewer = function() {
		if (self.selectedSource()) {
			self.isSelected(!self.isSelected());
			if (self.isSelected()) {
				model.numberOfViewersSelected(model.numberOfViewersSelected() + 1);
			} else {
				model.numberOfViewersSelected(model.numberOfViewersSelected() - 1);
			}
		}
	}
	self.drawPreview = function() {
		if (self.shouldDraw) {
			var leftOffset = (214 - self.previewImage.width) / 2;
			self.context.clearRect(0, 0, 214, 120);
			self.context.drawImage(self.previewImage, leftOffset, 0);
			self.shouldDraw = false;
			self.shouldFetch = true;
		}
		
		self.animationRequestID = window.requestAnimationFrame(self.drawPreview);
	}
	self.startPreview = function() {
		self.context = $('canvas')[model.viewers.indexOf(self)].getContext('2d');
		self.animationRequestID = requestAnimationFrame(self.drawPreview);
		self.shouldFetch = true;
	}
	self.stopPreview = function() {
		cancelAnimationFrame(self.animationRequestID);
		self.animationRequestID = null;
		self.previewImage = null;
		self.shouldFetch = false;
		self.context.clearRect(0, 0, 214, 120);
	}
	self.selectedSource.subscribe(function(oldValue) {
		if (oldValue) {
			self.stopPreview();
		}
	}, null, "beforeChange");
	self.selectedSource.subscribe(function(newValue) {
		if (newValue) {
			for (var i = 0; i < model.viewers().length; i++) {
				if (model.viewers()[i] != self && model.viewers()[i].selectedSource() == newValue) {
					model.viewers()[i].selectedSource(null);
				}
			}
			self.startPreview();
		}
	});
}

function Source(inData) {
	var self = this;
	self.uniqueID = inData.unique_id;
	self.displayName = inData.display_name;
	self.deviceName = inData.device_name;
	self.description = inData.description;
	self.isRecording = ko.observable(inData.is_recording);
	self.isPaused = ko.observable(inData.is_paused);
	self.lastWarning = ko.observable(null);
	self.selectedDestinations = ko.observableArray([]);
	
	self.selectedDestinationsLabel = ko.computed(function() {
		if (self.selectedDestinations().length) {
			for(var i = 0; i < model.destinations().length; i++) {
				var destination = model.destinations()[i];
				for(var j = 0; j < self.selectedDestinations().length; j++) {
					if (destination.uniqueID === self.selectedDestinations()[j]) {
						return destination.name;
					}
				}
			}
		}
		return "Destinations...";
	});
	self.hasWarning = ko.computed(function() {
		return (self.lastWarning() != null) ? true : false;
	});
}

function Destination(inData) {
	var self = this;
	self.uniqueID = inData.unique_id;
	self.name = inData.name;
	self.path = inData.path;
}

model = new MRRemoteModel();

function fetchPreview() {
	for (var i = 0; i < model.viewers().length; i++) {
		var viewer = model.viewers()[i];
		if (viewer.shouldFetch) {
			viewer.shouldFetch = false;
			viewer.previewImage = new Image();
			viewer.previewImage.onload = function(loadedViewer) {
				return function() {
					if (loadedViewer.selectedSource()) {
						if (loadedViewer.previewImage.width) {
							loadedViewer.shouldDraw = true;
						} else {
							loadedViewer.shouldFetch = true;
						}
					}
				}
			}(viewer);
			viewer.previewImage.src = '/sources/'+viewer.selectedSource().uniqueID+'/thumbnail?time='+Date.now();
		}
	}
	
	window.setTimeout(fetchPreview, 1000/30);
}

function updateViewerInfo(index) {
	if (index == model.viewers().length) {
		window.setTimeout(updateViewersInfo, 1000/60);
	} else {
		var source = model.viewers()[index].selectedSource();
		if (source) {
			$.getJSON('/sources/'+source.uniqueID+'/info', function(data) {
				var canvaswrap = $('.canvaswrap').eq(index);
				var vuMeters = canvaswrap.find('.vuMeterOverlay');
				for (var i = 0; i < 4; i++) {
					var height = 100;
					if (i < data.vu_meter_levels.length) {
						height = (1 - data.vu_meter_levels[i]) * 100;
					}
					vuMeters.eq(i).animate({height: height+"%"}, {duration: 20});
				}
				canvaswrap.find('.timecode').eq(0).text(data.timecode);
				source.isRecording(data.is_recording);
				source.isPaused(data.is_paused);
				source.lastWarning((data.last_warning) ? data.last_warning : null);
			}).always(function() {
				updateViewerInfo(index + 1);
			});
		} else {
			updateViewerInfo(index + 1);
		}
	}
}

function updateViewersInfo() {
	updateViewerInfo(0);
}

// Fetch sources and display them
$.ajax('/sources')
.done(function(inSources) {
	$.each(inSources, function(index, inSource) {
		if (inSource.is_enabled) {
			var source = new Source(inSource);
			model.sources.push(source);
			$.ajax('/sources/'+source.uniqueID+'/destinations').done(function(inSelectedDestinations) {
				source.selectedDestinations(inSelectedDestinations);
				source.selectedDestinations.subscribe(function(newValue) {
					if (newValue) {
						$.ajax({url: '/sources/'+source.uniqueID+'/destinations', type: 'PUT', data: JSON.stringify(newValue), contentType: 'application/json'});
					}
				});
			});
		}
	});
	updateViewersInfo();
});

// Fetch destinations and display them
$.ajax('/destinations')
.done(function(receivedDestinations) {
	$.each(receivedDestinations, function(index, inDestination) {
		model.destinations.push(new Destination(inDestination));
	});
});

$('#destinationlist').hide();

$('#topmenu').on("click", "div", function() {
	if ($(this).hasClass('selected') === false) {
		$('#topmenu div').toggleClass('selected');
		$('#sourcelist, #destinationlist').toggle();
	}
});

$('#bottom').on("click", ".stopButton", function() {
	for (var i = 0; i < model.viewers().length; i++) {
		if (model.viewers()[i].isSelected() && model.viewers()[i].selectedSource().isRecording()) {
			$.ajax('/sources/'+model.viewers()[i].selectedSource().uniqueID+'/stop');
		}
	}
}).on("click", ".recordButton", function() {
	for (var i = 0; i < model.viewers().length; i++) {
		if (model.viewers()[i].isSelected() && (model.viewers()[i].selectedSource().isRecording() === false || model.viewers()[i].selectedSource().isPaused())) {
			var action = (model.viewers()[i].selectedSource().isPaused()) ? 'resume' : 'record';
			$.ajax('/sources/'+model.viewers()[i].selectedSource().uniqueID+'/'+action);
		}
	}
}).on("click", ".pauseButton", function() {
	for (var i = 0; i < model.viewers().length; i++) {
		if (model.viewers()[i].isSelected() && model.viewers()[i].selectedSource() && model.viewers()[i].selectedSource().isRecording() && model.viewers()[i].selectedSource().isPaused() === false) {
			$.ajax('/sources/'+model.viewers()[i].selectedSource().uniqueID+'/pause');
		}
	}
}).on("click", ".timerButton", function() {
	var offset = $(this).offset();
	var timerPopOver = $("#timerPopOver");
	timerPopOver.css('left', 'calc('+offset.left+'px + 12px - '+(timerPopOver.outerWidth() / 2)+'px)').css('top', 'calc('+offset.top+'px - '+timerPopOver.outerHeight()+'px - 14px)').show();
});

$('#timerPopOver').on("click", ".apply", function() {
	if ($('#timerDuration:checked').val()) {
		var duration = $('#duration').val().split(':');
		for (var i = 0; i < model.viewers().length; i++) {
			if (model.viewers()[i].selectedSource()) {
				$.ajax('/sources/'+model.viewers()[i].selectedSource().uniqueID+'/record?duration='+(parseInt(duration[0], 10) * 3600 + parseInt(duration[1], 10) * 60));
				break;
			}
		}
		$('#timerPopOver').hide();
	} else if ($('#timerDate:checked').val()) {
		var endDateAsString = $('#endDate').val();
		if (endDateAsString.length) {
			var endDate = new Date(endDateAsString);
			for (var i = 0; i < model.viewers().length; i++) {
				if (model.viewers()[i].selectedSource()) {
					$.ajax('/sources/'+model.viewers()[i].selectedSource().uniqueID+'/record?end_date='+endDate.toJSON());
					break;
				}
			}
			$('#timerPopOver').hide();
		}
	}
}).on("click", ".cancel", function() {
	$('#timerPopOver').hide();
});

$('#viewers').on("click", ".destinationsLabel", function() {
	$(this).next().css('visibility', 'visible');
}).on("click", "button", function() {
	$(this).parent().css('visibility', 'hidden');
});

$('#gangRecording').on("click", "button", function() {
	for (var i = 0; i < model.viewers().length; i++) {
		if (model.viewers()[i].selectedSource()) {
			$.ajax({url: '/sources/'+model.viewers()[i].selectedSource().uniqueID+'/recording_name', type: 'PUT', data: JSON.stringify({recording_name: $('#gangRecording input').val()}), contentType: 'application/json'});
		}
	}
});

for(var i = 0; i < 9; i++) {
	model.viewers.push(new Viewer());
}

ko.applyBindings(model);
fetchPreview();

var socket = new WebSocket("ws://"+window.location.hostname+":8080/remote", "main_update.movierecorder.softronmedia.com/1.0");
socket.onmessage = function (event) {
	console.log(event.data);
	
	var message = JSON.parse(event.data);
	switch (message.dataType) {
		case "source":
			switch (message.change) {
				case "insertion":
					$.each(message.data, function(index, inSource) {
						if (inSource.is_enabled) {
							var source = new Source(inSource);
							model.sources.push(source);
							$.ajax('/sources/'+source.uniqueID+'/destinations').done(function(inSelectedDestinations) {
								source.selectedDestinations(inSelectedDestinations);
								source.selectedDestinations.subscribe(function(newValue) {
									if (newValue) {
										$.ajax({url: '/sources/'+source.uniqueID+'/destinations', type: 'PUT', data: JSON.stringify(newValue), contentType: 'application/json'});
									}
								});
							});
						}
					});
					break;
				case "removal":
					for (var i = 0; i < message.data.length; i++) {
						model.sources.remove(function(source) {
							return (source.uniqueID === message.data[i]);
						});
					}
					break;
				case "replacement":
					var source = new Source(message.data);
					for (var i = 0; i < model.sources().length; i++) {
						if (model.sources()[i].uniqueID == source.uniqueID) {
							// Replace any property that changed
							/*if (typeof source.is_enabled !== 'undefined') {
								model.sources()[i].isEnabled(source.is_enabled);
							}*/
							if (typeof source.display_name !== 'undefined') {
								model.sources()[i].displayName(source.display_name);
							}
							if (typeof source.description !== 'undefined') {
								model.sources()[i].description(source.description);
							}
							if (typeof source.is_recording !== 'undefined') {
								model.sources()[i].isRecording(source.is_recording);
							}
							if (typeof source.is_paused !== 'undefined') {
								model.sources()[i].isPaused(source.is_paused);
							}
							
							break;
						}
					}
					break;
			}
			break;
		case "destination":
			switch (message.change) {
				case "insertion":
					$.each(message.data, function(index, inDestination) {
						model.destinations.push(new Destination(inDestination));
					});
					break;
				case "removal":
					for (var i = 0; i < message.data.length; i++) {
						model.destinations.remove(function(destination) {
							return (destination.uniqueID === message.data[i]);
						});
					}
					break;
				case "replacement":
					var destination = new Destination(message.data);
					for (var i = 0; i < model.destinations().length; i++) {
						if (model.destinations()[i].uniqueID == destination.uniqueID) {
							model.destinations.replace(model.destinations()[i], destination);
							break;
						}
					}
					break;
			}
			break;
	}
	
}
